package output;

use strict;
require 'one_dimensional.pm';

use vars qw ($data $config);

*data = *main::data;
*config = *main::config;

print "Generating output with top10s (simple) <output/top10.pm>\n" if $main::debug;

sub generate
{
	my $max = 10;
	my @key1s = sort (keys %{$data});
	my $directory = '';
	if ($config->{'directory'})
	{
		$directory = $config->{'directory'};
		$directory =~ s/\/$//;
		if (!-d $directory) { die "Directory \"$directory\" does not exist\n"; }
		$directory .= '/';
	}

	open (HTMLFILE, '> ' . $directory . 'index.html');
	print HTMLFILE html_head ('top' . $max);
	foreach my $key1 (@key1s)
	{
		print HTMLFILE "<a name=\"$key1\">\n";
		if ($config->{'draw_graph'})
		{       
			my $counter;
			require GD::Graph::bars;
			my (@data, @tmp1, @tmp2);
			foreach (sort { $data->{$key1}{$b} <=> $data->{$key1}{$a} }keys %{$data->{$key1}})
			{
				$counter++;
				if ($counter <= $max)
				{
					my $tmp = $_;
					if (length ($tmp) > 24) { $tmp = substr ($tmp, 0, (24 - 2)) . '..'; }
					push (@tmp1, $tmp);
					push (@tmp2, $data->{$key1}{$_});
				}
			}
			@data = (\@tmp1, \@tmp2);
			create_graph ($key1, \@data, $directory);
			print HTMLFILE "<img src=\"$key1.png\" width=\"600\" height=\"250\" border=\"0\">\n\n";
		}
		my $counter;
		print HTMLFILE "<table border=\"0\" cellspacing=\"1\" width=\"70\%\">\n<tr><th colspan=\"3\" bgcolor=\"$config->{'color'}{'header'}\">$key1</th></tr>\n";
		my @val1s;
		if (scalar (keys %{$data->{$key1}}) > $max) { @val1s = (sort { $data->{$key1}{$b} <=> $data->{$key1}{$a} } ((keys %{$data->{$key1}})))[0..($max - 1)]; }
		else { @val1s = (sort { $data->{$key1}{$b} <=> $data->{$key1}{$a} } ((keys %{$data->{$key1}}))); }
		foreach my $val1 (@val1s)
		{
			$counter++;
			my $general = $data->{$key1}{$val1};
			print HTMLFILE "<tr><td bgcolor=\"$config->{'color'}{'header'}\" width=\"5\%\">$counter</td><td bgcolor=\"$config->{'color'}{'subhdr'}\" width=\"80\%\">$val1</td><td bgcolor=\"$config->{'color'}{'normal'}\" width=\"15\%\">$data->{$key1}{$val1}</td></tr>\n";
		}
		print HTMLFILE "</table>\n<br><br>\n\n";
	}
	print HTMLFILE html_foot();
	close HTMLFILE;
}

sub create_graph
{       
	my %config = (  'width'         =>      600,
			'height'        =>      250,
			'max-length'    =>      24      
							);
	my $key1 = $_[0];
	my @graph_data = @{$_[1]};
	my $directory = $_[2];
	die "No input! (\$key1)\n" if !$key1;
	if (!@graph_data) { die "No input! (\@graph_data)\n"; }
	my $graph = GD::Graph::bars->new($config{'width'}, $config{'height'});
	$graph->set (   x_label                 =>      '',
			y_label                 =>      'Requests',
			title                   =>      $key1,
			y_min_value             =>      0,
			long_ticks              =>      1,
			y_tick_number           =>      5,
			bgclr                   =>      'white',
			fgclr                   =>      'black',
			dclrs                   =>      [ qw(lgray gray dgray) ],
			borderclrs              =>      [ qw(black black white) ],
			x_labels_vertical       =>      1,
			shadowclr               =>      'gray',
			shadow_depth            =>      3,
			labelclr                =>      'black',
			axislabelclr            =>      'black',
			legendclr               =>      'black',
			valuesclr               =>      'black',
			textclr                 =>      'black',
			cycle_clrs              =>      0
		);
	open (IMAGEFILE, '> ' . $directory . $key1 . '.png');
	print IMAGEFILE $graph->plot(\@graph_data)->png();
	close IMAGEFILE;
}

sub html_head
{
	my $cathegory = $_[0];
	if (!$cathegory) { $cathegory = 'General'; }
	my $text;
	$text = "<html>\n<head>\n<title>$main::VERSION: $cathegory";
	$text .= "</title>\n</head>\n\n<body bgcolor=\"$config->{'color'}{'bgcolor'}\" text=\"$config->{'color'}{'text'}\" link=\"$config->{'color'}{'links'}\" vlink=\"$config->{'color'}{'links'}\" alink=\"$config->{'color'}{'links'}\">\n";
	$text .= "<h1><img src=\"logo.gif\" width=\"32\" height=\"32\" border=\"0\"> $main::VERSION: $cathegory</h1>\n\n<a name=\"beginofpage\">\n<p>";
	foreach my $key1 (sort (keys %$data))
	{
		$text .= "[ <a href=\"#$key1\">$key1</a> ] ";
	}
	$text .= "</p>\n";
	return $text;
}

sub html_foot
{
	my $text = "<hr>\n<p align=\"right\"><font size=\"2\">generated with \&quot;$main::VERSION\&quot;<br>\nwritten by $main::AUTHOR \&lt;<a href=\"mailto:$main::EMAIL\">$main::EMAIL</a>\&gt;<br>\nget the newest version from <a href=\"$main::HOMEPAGE\" target=\"_blank\">$main::HOMEPAGE</a></font></p>\n";
	$text .= "</body>\n</html>\n";
	return $text;
}

sub to_bytes
{
	my $bytes = $_[0];
	if ($bytes > 1024**3) { $bytes = (int (10 * ($bytes / 1024**3)) /10) . 'Gb'; }
	elsif ($bytes > 1024**2) { $bytes = (int (10 * ($bytes / 1024**2)) /10) . 'Mb'; }
	elsif ($bytes > 1024) { $bytes = (int (10 * ($bytes / 1024)) /10) . 'kb'; }
	else { $bytes .= 'b'; }
	return $bytes;
}
